
local MAJOR = "StarLibEvaluator-1.0" 
local MINOR = 1
assert(LibStub, MAJOR.." requires LibStub") 
local LibEvaluator = LibStub:NewLibrary(MAJOR, MINOR)
if not LibEvaluator then return end
local LibError = LibStub("StarLibError-1.0")
assert(LibError, "LibEvaluator requires StarLibError-1.0")
local error = LibError:New("Evaluator")

local pool = setmetatable({}, {__mode = "k"})
	
if not LibEvaluator.__index then
	LibEvaluator.__index = LibEvaluator
end

-- Create a new StarLibEvaluator
-- @name StarLibEvaluator.New
-- @usage LibEvaluator:New()
-- @param errorLevel The errorLevel for this object
-- @return A new StarLibEvaluator object
function LibEvaluator:New(environment, errorLevel) 	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end

	setmetatable(obj, self)

	obj.environment = environment
	obj.errorLevel = errorLevel
	obj.error = LibError:New(MAJOR, errorLevel)
	
	return obj	
end

-- Delete a StarLibEvaluator object
-- @name LibEvaluator.Del
-- @usage LibEvaluator:Del(evaluator) or object:Del()
-- @param ev A StarLibEvaluator object
-- @return Nothing

function LibEvaluator:Del(ev)
	if not ev then ev = self end
	ev.error:Del()
	pool[ev] = true
end

-- Validate some code
-- @name LibEvaluator.Validate
-- @usage LibEvaluator:Validate(str)
-- @param str The code to validate
-- @return True if the code validated, false otherwise
function LibEvaluator:Validate(environment, str)
	local ret, err = self.ExecuteCode(environment, "validate", str)
						
	if not ret then
		self.error:Print(("Code failed to load. Error message: %s"):format(type(err) == "string" and err or "???"))
		v.error = true
		return false
	end
	self.error:Print(("Code loaded without error. Return value: %s"):format(type(ret) == "string" and ret or "???"))
	return true
end


do 
	local function errorhandler(str)
		error:Print(str)
	end
	
	local cache = {} --setmetatable({},{__mode='v'})	
	
	-- Execute some code
	-- @name LibEvaluator.Execute
	-- @usage LibEvaluator.ExecuteCode(self, tag, code, dontSandbox, defval)
	-- @param self The LibEvaluator object or some other enironment. This is the environment for your script.
	-- @param tag A name for your runnable
	-- @param dontSandbox Whether to sandbox the execution or not
	-- @param defval The default value if any
	-- @return ret1, ret2, ret3 -- return values from your code. You can pass 3 return values back from your script.
	LibEvaluator.ExecuteCode = function(self, tag, code, dontSandbox, defval)
		if not defval and not dontDefault then defval = "" end
		
		assert(self, "LibEvaluator.ExecuteCode requires a self object")
		assert(tag, "LibEvaluator.ExecuteCode requires a tag")
		
		code = code or "return nil"
		
		code = "return function() " .. code .. " end"
				
		local runnable = cache[code]
		local err
		
		if dontSandbox then runnable = nil end
				
		if not runnable then
		
			runnable, err = loadstring(code, tag)
	
			if type(runnable) ~= "function" then 
				error:Print("Error in runnable: " .. err)
				return nil
			end
						
			if not dontSandbox then
				self.self = self -- note that using this reference is deprecated.
				self._G = _G
				self.select = select
				self.format = format
		
				setfenv(runnable, self)
			end
			
			cache[code] = runnable(xpcall, errorhandler)
			
			runnable = cache[code]	
		end
		
		local ret1, ret2, ret3 = runnable()
								
		if not ret1 then return defval end
		
		return ret1, ret2, ret3
	end
end